<?php

/**
 * @file
 * This is the code file for the main form for sending the messages.
 */

 /* ********************************
 *
 * Functions for sending a message.
 *
 * *******************************/

/**
 * The mail page
 *
 * @return
 *   Either an error, if flood control is active and triggered, or a
 *   rendered form.
 */
function mass_contact_site_page() {
  if (!user_access('mass contact administer') && !flood_is_allowed('mass_contact', variable_get('mass_contact_hourly_threshold', 3))) {
    $output = t("You cannot send more than %number messages per hour. Please try again later.", array('%number' => variable_get('mass_contact_hourly_threshold', 3)));
  }
  else {
    $output = drupal_get_form('mass_contact_mail_page');
  }

  return $output;
} // End of mass_contact_site_page().


/**
 * Generates the main Mass Contact mail form.
 *
 * @param form_state
 *   A keyed array containing the current state of the form.
 *
 * @return
 *   An associative array that defines the form to be built.
 */
function mass_contact_mail_page($form, $form_state) {
  global $user;

  $form = array();
  $categories = array();
  $default_category = array();
  $default_category_name = '';

  $result = db_select('mass_contact', 'mc')
    ->fields('mc', array('cid', 'category', 'selected'))
    ->orderBy('weight', 'ASC')
    ->orderBy('category', 'ASC')
    ->execute();
  foreach ($result as $category) {
    if (user_access('mass contact send to users in the ' . $category->category . ' category')) {
      $categories[$category->cid] = $category->category;

      if ($category->selected) {
        $default_category[] = $category->cid;
        $default_category_name = $category->category;
      }
    }
  }

  if (count($categories) == 1) {
    $default_category = array_keys($categories);
    $default_category_name = $categories[$default_category[0]];
  }

  if (count($categories) > 0) {
    $form['#token'] = $user->name . $user->mail;
    $form['contact_information'] = array(
      '#markup' => filter_xss_admin(variable_get('mass_contact_form_information', t('Send an e-mail message using the contact form below.'))),
    );

    ////////////////////////////////////////////////////////////
    // Add the field for specifying the sender's name.
    $mass_contact_default_sender_name = variable_get('mass_contact_default_sender_name', '');
    if ($mass_contact_default_sender_name) {
      if (user_access('mass contact change default sender information')) {
        $form['name'] = array(
          '#type' => 'textfield',
          '#title' => t('Your name'),
          '#maxlength' => 255,
          '#default_value' => $mass_contact_default_sender_name,
          '#required' => TRUE,
        );
      }
      else {
        $form['name'] = array(
          '#type' => 'item',
          '#title' => t('Your name'),
          '#markup' => $mass_contact_default_sender_name,
        );
      }
    }
    else {
      $form['name'] = array(
        '#type' => 'textfield',
        '#title' => t('Your name'),
        '#maxlength' => 255,
        '#default_value' => $user->uid ? $user->name : '',
        '#required' => TRUE,
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying the sender's e-mail address.
    $mass_contact_default_sender_email = variable_get('mass_contact_default_sender_email', '');
    if ($mass_contact_default_sender_email) {
      if (user_access('mass contact change default sender information')) {
        $form['mail'] = array(
          '#type' => 'textfield',
          '#title' => t('Your e-mail address'),
          '#maxlength' => 255,
          '#default_value' => $mass_contact_default_sender_email,
          '#required' => TRUE,
        );
      }
      else {
        $form['mail'] = array(
          '#type' => 'item',
          '#title' => t('Your e-mail address'),
          '#markup' => $mass_contact_default_sender_email,
        );
      }
    }
    else {
      $form['mail'] = array(
        '#type' => 'textfield',
        '#title' => t('Your e-mail address'),
        '#maxlength' => 255,
        '#default_value' => $user->uid ? $user->mail : '',
        '#required' => TRUE,
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying the category(ies).
    if ((count($categories) > 1) || !isset($default_category)) {
      // Display a choice when one is needed.
      $form['cid'] = array(
        '#type' => 'select',
        '#title' => t('Category'),
        '#default_value' => $default_category,
        '#options' => $categories,
        '#required' => TRUE,
        '#multiple' => TRUE,
      );
    }
    else {
      // Otherwise, just use the default category.
      $form['cid'] = array(
        '#type' => 'value',
        '#value' => $default_category,
      );
      $form['cid-info'] = array(
        '#type' => 'item',
        '#title' => t('Category'),
        '#markup' => '<p>This message will be sent to all users in the ' . $default_category_name . ' category.</p>',
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying whether opt-outs are respected or not.
    $optout_setting = variable_get('mass_contact_optout_d', 0);

    // Allow users to opt-out of mass e-mails:
    // 0 => 'No', 1 == 'Yes', 2 == 'Selected categories'.
    if ($optout_setting == 1 || $optout_setting == 2) {
      // Allow to override or respect opt-outs if admin, otherwise use default.
      if (user_access('mass contact administer')) {
        $form['optout'] = array(
          '#type' => 'checkbox',
          '#title' => t('Respect user opt-outs.'),
          '#default_value' => 1,
        );
      }
      else {
        $form['optout'] = array(
          '#type' => 'hidden',
          '#default_value' => 1,
        );
      }
    }
    else {
      $form['optout'] = array(
        '#type' => 'hidden',
        '#default_value' => 0,
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying whether the recipients are in the To or
    // BCC field of the message.
    // Check if the user is allowed to override the BCC setting.
    if (user_access('mass contact override bcc')) {
      $form['bcc'] = array(
        '#type' => 'checkbox',
        '#title' => t('Send as BCC (hide recipients)'),
        '#default_value' => variable_get('mass_contact_bcc_d', 1),
      );
    }
    // If not, then just display the BCC info.
    else {
      $form['bcc'] = array(
        '#type' => 'value',
        '#value' => variable_get('mass_contact_bcc_d', 1),
      );
      $form['bcc-info'] = array(
        '#type' => 'item',
        '#title' => t('Send as BCC (hide recipients)'),
        '#markup' => '<p>' . (variable_get('mass_contact_bcc_d', 1) ? t('Recipients will be hidden from each other.') : t('Recipients will NOT be hidden from each other.')) . '</p>',
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying the subject of the message.
    $form['subject'] = array(
      '#type' => 'textfield',
      '#title' => t('Subject'),
      '#maxlength' => 255,
      '#required' => TRUE,
    );

    ////////////////////////////////////////////////////////////
    // Add the field for specifying the body and text format of the message.
    //
    // Get the HTML input format setting and the corresponding name.
    if (module_exists('mimemail')) {
      // Get the admin specified default text format.
      $specified_filter_format = variable_get('mass_contact_html_format', filter_default_format($user));

      // The admin has not specified one, there wou'nt be a match for the
      // default.
      if (!empty($specified_filter_format)) {
        // Get all the text formats the user is allowed access to.
        $filter_formats = filter_formats($user);
        // Cycle through the list of possible text formats.
        foreach ($filter_formats as $filter_format) {
          // If the current filter format matches the admin specified one...
          if ($filter_format->format == $specified_filter_format['format'] || $filter_format->format == $specified_filter_format) {
            // ...save it for later use.
            $default_filter_format = $filter_format;
          }
        }
      }

      // If nothing was selected above, set the default to the Drupal system
      // default.
      if (empty($default_filter_format)) {
        $default_filter_format = $filter_formats[filter_fallback_format()];
      }

      // Check if the user is allowed to override the text format.
      if (user_access('mass contact override text format')) {
        // The user is allowed to override the text format, so display a
        // textarea that has the text format selection field attached.
        $form['body']['message'] = array(
          '#type' => 'text_format',
          '#title' => t('Message'),
          '#format' => !empty($default_filter_format->format) ? $default_filter_format->format : NULL,
          '#rows' => 12,
          '#required' => TRUE,
        );
      }
      else {
        // The user is not allowed to override the text format, so display a
        // textarea that does not have the text format selection field
        // attached.
        $form['body']['message'] = array(
          '#type' => 'textarea',
          '#title' => t('Message'),
          '#rows' => 12,
          '#required' => TRUE,
        );
        // Save the text format information for later use.
        $form['body_filter_format'] = array(
          '#type' => 'value',
          '#value' => $default_filter_format,
        );
        // Display text format information to the user.
        $form['body_filter_format_info'] = array(
          '#type' => 'item',
          '#description' => '<p>' . (variable_get('mass_contact_html_d', 1) ? t('The message will be sent using the %filter text format. More information about what is available is on the <a href="@formats_descriptions">Compose tips</a> page.', array('%filter' => $default_filter_format->name, '@formats_descriptions' => url('filter/tips'))) : t('The message will be sent as plain text.')) . '</p>',
        );
      }

      ////////////////////////////////////////////////////////////
      // If the user has access, add the field for specifying the attachment.
      if (user_access('mass contact include attachments')) {
        for ($i = 1; $i <= variable_get('mass_contact_number_of_attachments', '3'); $i++) {
          $form['attachment_' . $i] = array(
            '#type' => 'file',
            '#title' => t('Attachment #!number', array('!number' => $i)),
          );
        }
      }
    }
    else {
      // Mime Mail is not installed, so display a textarea that does not have
      // the text format selection field attached, as this message will only
      // be sent as plain text.
      $form['body']['message'] = array(
        '#type' => 'textarea',
        '#title' => t('Message'),
        '#rows' => 12,
        '#required' => TRUE,
      );
      // Save the text format information for later use.
      $form['body_filter_format'] = array(
        '#type' => 'value',
        '#value' => filter_format_load('plain_text'),
      );
      // Display text format information to the user.
      $form['body_filter_format_info'] = array(
        '#type' => 'item',
        '#description' => '<p>' . t('The message will be sent as plain text.') . '</p>',
      );
    }

    ////////////////////////////////////////////////////////////
    // We do not allow anonymous users to send themselves a copy because it
    // can be abused to spam people.
    if ($user->uid) {
      $form['copy'] = array(
        '#type' => 'checkbox',
        '#title' => t('Send yourself a copy.'),
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the field for specifying whether to save the message as a node or
    // not.
    if (user_access('mass contact archive messages')) {
      // Check if the user is allowed to override the node copy setting.
      if (user_access('mass contact override archiving')) {
        $form['nodecc'] = array(
          '#type' => 'checkbox',
          '#title' => t('Archive a copy of this message on this website'),
          '#default_value' => variable_get('mass_contact_nodecc_d', 1),
        );
      }
      // If not, then do it or not based on the administrative setting.
      else {
        $form['nodecc'] = array(
          '#type' => 'hidden',
          '#default_value' => variable_get('mass_contact_nodecc_d', 1),
        );
        $form['nodecc'] = array(
          '#type' => 'item',
          '#title' => t('Archive a copy of this message on this website'),
          '#markup' => t('A copy of this message will !not be archived on this website.', array('!not' => variable_get('mass_contact_nodecc_d', 1) ? '' : 'not')),
        );
      }
    }
    // If not, then do it or not based on the administrative setting.
    else {
      $form['nodecc'] = array(
        '#type' => 'hidden',
        '#default_value' => variable_get('mass_contact_nodecc_d', 1),
      );
      $form['nodecc'] = array(
        '#type' => 'item',
        '#title' => t('Archive a copy of this message on this website'),
        '#markup' => t('A copy of this message will !not be archived on this website.', array('!not' => variable_get('mass_contact_nodecc_d', 1) ? '' : 'not')),
      );
    }

    ////////////////////////////////////////////////////////////
    // Add the submit button.
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Send e-mail'),
    );
  }
  else {
    $form['error'] = array(
      '#markup' => '<p><b>' . t('Either you have not created any categories, or you are not allowed to send to any of the existing categories.') . '<br /><br />' . t('Either create at least one category of users to send to, or contact your system administer for access to the existing categories.') . '</b>',
    );
  }

  if (user_access('mass contact administer')) {
    $form['tasklist'] = array(
      '#type' => 'fieldset',
      '#title' => t('Related tasks'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#prefix' => '<p>',
    );
    $form['tasklist']['list'] = array(
      '#markup' => '<p><ol>' .
        '<li>' . l(t('Set Permissions'), 'admin/user/permissions', array('fragment' => 'module-mass_contact')) . '</li>' .
        '<li>' . l(t('List current categories'), 'admin/config/system/mass_contact') . '</li>' .
        '<li>' . l(t('Add new category'), 'admin/config/system/mass_contact/add') . '</li>' .
        '<li>' . l(t('Configure the module'), 'admin/config/system/mass_contact/settings') . '</li>' .
        '<li>' . l(t('Help'), 'admin/help/mass_contact') . '</li></ol>',
    );
  }

  return $form;
} // End of mass_contact_mail_page().


/**
 * Validates the main Mass Contact mail form.
 *
 * @param form
 *   An associative array containing the structure of the form.
 * @param form_state
 *   A keyed array containing the current state of the form.
 */
function mass_contact_mail_page_validate($form, &$form_state) {
  if (!$form_state['values']['cid']) {
    form_set_error('category', t('You must select a valid category.'));
  }

  if (user_access('mass contact change default sender information') == 1 && !valid_email_address($form_state['values']['mail'])) {
    form_set_error('mass_contact', t('You must enter a valid e-mail address.'));
  }
} // End of mass_contact_mail_page_validate().


/**
 * Processes the main Mass Contact mail form.
 *
 * @param form
 *   An associative array containing the structure of the form.
 * @param form_state
 *   A keyed array containing the current state of the form.
 */
function mass_contact_mail_page_submit($form, &$form_state) {
  // Initialize some variables.
  $params = array();
  $bcc = $form_state['values']['bcc'];
  $nodecc = $form_state['values']['nodecc'];
  $message = '';
  $send_error = 0;

  // Set the sender's name.
  $default_sender_name = variable_get('mass_contact_default_sender_name');
  if (!empty($form_state['values']['name'])) {
    $from_name = $form_state['values']['name'];
  }
  elseif (!empty($default_sender_name)) {
    $from_name = $default_sender_name;
  }
  else {
    $from_name = variable_get('site_name');
  }

  // Set the sender's e-mail address.
  $default_sender_email = variable_get('mass_contact_default_sender_email');
  if (!empty($form_state['values']['mail'])) {
    $from_email = $form_state['values']['mail'];
  }
  elseif (!empty($default_sender_email)) {
    $from_email = $default_sender_email;
  }
  else {
    $from_email = variable_get('site_mail');
  }


  // Prepare the message header. This currently only sets the character set.
  $params = _mass_contact_prepare_header();

  // Prepare the message body part(s).
  $message = _mass_contact_prepare_bodies($form_state['values']);

  // For the node copy, we want to handle attachments differently, so we'll
  // save the message as it currently stands.
  if (variable_get('mass_contact_nodecc_d', 1) || ($nodecc == 1)) {
    $node_message = $message;
  }


  // If there are attachments, add them.
  if (module_exists('mimemail')) {
    for ($i = 1; $i <= variable_get('mass_contact_number_of_attachments', '3'); $i++) {
      if (!empty($_FILES['files']['size']['attachment_' . $i])) {
        $params['attachments'] = _mass_contact_process_mime_mail_attachments($nodecc);
        break;
      }
    }
  }


  // Add the message to the main variable.
  $params['body'] = $message;


  // Load the category information.
  $cids = array();
  foreach ($form_state['values']['cid'] as $cid) {
    $cids[] = $cid;
  }

  // Select the chosen categories from the database.
  $categories = db_select('mass_contact', 'mc')
    ->condition('cid', $cids, 'IN')
    ->fields('mc')
    ->execute();

  $everyone = FALSE;
  $message_subject = '';

  // Check to see if the authenticated users role was specified in any
  // category.
  foreach ($categories as $category) {
    // Get the list of roles which were chosen through the categry selection.
    $roles = explode(',', $category->recipients);
    foreach ($roles as $role) {
      // Role 2 is the authenticated users role, which means everyone. This
      // needs to be checked for first. Otherwise, duplicate messages could be
      // sent.
      if ($role == 2) {
        $everyone = TRUE;

        if (variable_get('mass_contact_category_override', 1)) {
          $message_subject = t('[@category] @subject', array('@category' => $category->category, '@subject' => $form_state['values']['subject']));
        }

        break 2;
      }
    }
  }

  // Get the list of users who've opted out of this mailing.
  $opt_out_users = array();

  if ($form_state['values']['optout']) {
    // Get the list of users who've opted out.
    $opt_out_users = _mass_contact_create_opt_out_list($categories);
  }


  // Create the recipient lists.

  // Initialize the array of recipients and subjects.
  $recipient_lists = array();

  // If the 'authentecated users' role was selected in any category, we ignore
  // all categories.
  if ($everyone) {
    // Set subject.
    if ($message_subject == '') {
      $message_subject = $form_state['values']['subject'];
    }

    // Create the recipient list, consisting of all users.
    $recipient_lists['All authenticated users'] = array(
      // Set the recipients.
      'recipients' => _mass_contact_create_recipient_list(array(2), $opt_out_users),
      // Set the subject.
      'subject' => $message_subject,
    );
  }
  // If the module is configured to not include the category in the subject,
  // combine all users in all categories/roles into a single listing of
  // recpients.
  elseif (!variable_get('mass_contact_category_override', 1)) {
    $roles = array();

    // Select the chosen categories from the database.
    $categories = db_select('mass_contact', 'mc')
      ->condition('cid', $cids, 'IN')
      ->fields('mc')
      ->execute();

    // Cycle through all the choosen categories.
    foreach ($categories as $category) {
      // Get the list of roles which were chosen through the categry selection.
      $roles = array_merge($roles, explode(',', $category->recipients));
    }

    // Create the recipient list.
    $recipient_lists['All categories combined'] = array(
      // Set the recipients.
      'recipients' => _mass_contact_create_recipient_list($roles, $opt_out_users),
      // Set the subject.
      'subject' => $form_state['values']['subject'],
    );
  }
  // Cycle through all categories and create groups of recipients.
  else {
    // Select the chosen categories from the database.
    $categories = db_select('mass_contact', 'mc')
      ->condition('cid', $cids, 'IN')
      ->fields('mc')
      ->execute();
    foreach ($categories as $category) {
      // Get the list of roles which were chosen through the categry selection.
      $roles = explode(',', $category->recipients);

      // Create the recipient lists, per category.
      $recipient_lists[$category->category] = array(
        // Set the recipients.
        'recipients' => _mass_contact_create_recipient_list($roles, $opt_out_users),
        // Set the subject.
        'subject' => t('[@category] @subject', array('@category' => $category->category, '@subject' => $form_state['values']['subject'])),
      );
    }
  }


  // Verify we have someone to send to.
  $no_recipients = FALSE;
  $recipients_count = 0;

  // Iterate through each recipient list to verify there are recipients
  // somewhere.
  if (!empty($recipient_lists)) {
    foreach ($recipient_lists as $recipient_list) {
      // Check for empty recipient list.
      if (count($recipient_list['recipients']) == 0) {
        $recipients_count++;
      }
    }
  }

  // There are categories selected, but all of them are empty.
  if (count($recipient_lists) == $recipients_count) {
    drupal_set_message(t('No users were in the selected category(ies), and therefore, the message was not sent.'));
    $no_recipients = TRUE;
    $form_state['redirect'] = '/';
  }

  if (!$no_recipients) {
    // We have recipients, so send the message to them.

    // The count of all e-mail addresses thie message is sent to.
    $total_recipients = 0;
    // An array of all categories and e-mail addresses thie message is sent to.
    $all_recipients = array();

    // Iterate through each recipient list.
    foreach ($recipient_lists as $category => $recipient_list) {
      // Only process this recipient list if it is not empty.
      if (count($recipient_list['recipients']) > 0) {
        // If there's a recipient limit, break up recipient list.
        $recipient_limit = variable_get('mass_contact_recipient_limit', 0);
        if (count($recipient_list['recipients']) > $recipient_limit && $recipient_limit != 0) {
          $recipient_count = 0;
          // The batch count.
          $batch = 0;
          // An array of the current grouping of categories and e-mail addresses thie
          // message is sent to.
          $current_recipients = array();

          foreach ($recipient_list['recipients'] as $recipient_name => $recipient_address) {
            //$current_recipients[] = $recipient_name . " <" . $recipient_address . ">";
            //$partial_list[] = $recipient_name . " <" . $recipient_address . ">";
            $current_recipients[] = $recipient_address;
            $partial_list[] = $recipient_address;
            $recipient_count = count($partial_list);

            if ($recipient_count == $recipient_limit) {
              // Increment the batch count.
              ++$batch;

              //  Finalizes the headers, by setting the To, Subject, and
              //  possibly, Bcc fields.
              $to = _mass_contact_finalize_headers($bcc, implode(', ', $partial_list), $params, $from_email, $recipient_list['subject']);

              // Send the message. Save the inverse of the result for tallying.
              $send_error += !(_mass_contact_send_message('mail_page_' . $batch, $to, $params, $from_email, '[Success] Sent batch #' . $batch . '.'));

              // Reset the variables.
              $partial_list = array();
              $recipient_count = 0;
            }
          }

          // Send the remainder.
          if ($recipient_count != 0) {
            //  Finalizes the headers, by setting the To, Subject, and
            //  possibly, Bcc fields.
            $to = _mass_contact_finalize_headers($bcc, implode(', ', $partial_list), $params, $from_email, $recipient_list['subject']);

            // Send the message. Save the inverse of the result for tallying.
            $send_error += !(_mass_contact_send_message('mail_page_last', $to, $params, $from_email, '[Success] Sent the remainder.'));
          }

          $total_recipients = count($current_recipients);
          $listed_recipients = implode(', ', $current_recipients);
          $all_recipients = array_merge($all_recipients, $current_recipients);
        }
        // There's no recipient limit, so send the message to all the recipients
        // at the same time.
        else {
          // An array of the current grouping of categories and e-mail addresses thie
          // message is sent to.
          $current_recipients = array();

          foreach ($recipient_list['recipients'] as $recipient_name => $recipient_address) {
            //$current_recipients[] = $recipient_name . " <" . $recipient_address . ">";
            $current_recipients[] = $recipient_address;
          }

          $total_recipients = count($current_recipients);
          $listed_recipients = implode(', ', $current_recipients);
          $all_recipients = array_merge($all_recipients, $current_recipients);

          //  Finalizes the headers, by setting the To, Subject, and possibly,
          //  Bcc fields.
          $to = _mass_contact_finalize_headers($bcc, implode(', ', $current_recipients), $params, $from_email, $recipient_list['subject']);

          // Send the message. Save the inverse of the result for tallying.
          $send_error += !(_mass_contact_send_message('mail_page', $to, $params, $from_email, '[Success] Sent all at once.'));
        }
      }

      // Archive the message as a node.
      if ($nodecc) {
        if (is_array($form_state['values']['message'])) {
          _mass_contact_save_node($params, $node_message, $form_state['values']['message']['format'], $from_email, $category, $to, !$send_error);
        }
        else {
          _mass_contact_save_node($params, $node_message, $form_state['values']['body_filter_format']->format, $from_email, $category, $to, !$send_error);
        }
      }
    }

    // If there were no errors, do the final operations.
    if (!$send_error) {
      // If the sender requested a copy, send it.
      if (!$no_recipients && $form_state['values']['copy'] && !$bcc) {
        if (!_mass_contact_send_message('user-copy', $from_email, $params, $from_email, 'A copy was sent to you.')) {
          // There was an error, so inform the user.
          if (!module_exists('help')) {
            drupal_set_message(t("A copy could not be sent to you. Check the logs for error messages. For more information, see the Troubleshooting secion of Mass Contact's !help page or README.txt.", array('!help' => l(t('help'), 'admin/help/mass_contact'))));
          }
          else {
            drupal_set_message(t("A copy could not be sent to you. Check the logs for error messages. For more information, see the Troubleshooting secion of Mass Contact's README.txt."));
          }
        }
      }

      // Register this message send to the flood control mechanism.
      flood_register_event('mass_contact');

      // Log the operation.
      watchdog('mass_contact', '%name-from sent a message to the %category group, which included these recipients: @e-mails.', array('%name-from' => $from_name . ' <' . $from_email . '>', '%category' => $category, '@e-mails' => $listed_recipients));

      // Prepare some variables for the printing of the final results.
      $total_recipients = count($all_recipients);
      $unique_addresses_count = count(array_unique($all_recipients));
      $listed_recipients = implode(', ', $all_recipients);

      // Print out the final results of the complete operation.
      if ($total_recipients == $unique_addresses_count) {
        drupal_set_message(t('Your message was successfully sent to a total of @total e-mail addresses: @e-mails', array('@total' =>  $total_recipients, '@e-mails' => $listed_recipients)));
      }
      else {
        drupal_set_message(t('Your message was successfully sent to a total of @total e-mail addresses, of which, there may have been !dups duplicates: @e-mails', array('@total' =>  $total_recipients, '!dups' => $total_recipients - $unique_addresses_count, '@e-mails' => $listed_recipients)));
      }
    }
    else {
      // Otherwise, inform the user that there were error(s).
      if (module_exists('help')) {
        drupal_set_message(t("@errors error(s) were encountered sending the message. Check the logs for error messages. For more information, see the Troubleshooting section of Mass Contact's !help page or README.txt.", array('@errors' => $send_error, '!help' => l(t('help'), 'admin/help/mass_contact'))));
      }
      else {
        drupal_set_message(t("@errors error(s) were encountered sending the message. Check the logs for error messages. For more information, see the Troubleshooting section of Mass Contact's README.txt.", array('@errors' => $send_error)));
      }
    }
  }

  // Redirect to the home page, rather than back to the mail page, to avoid
  // contradictory messages if flood control has been activated.
  $form_state['redirect'] = '/';
} // End of mass_contact_mail_page_submit().


/* *************************
 *
 * Private helper functions.
 *
 * ************************/


/**
 * Prepares the message's header.
 *
 * @param boundary_attachment
 *   A string containing boundary for the attachments.
 * @param boundary_html
 *   A string containing boundary for the HTML part.
 *
 * @return
 *   An array containing the prepared header.
 */
function _mass_contact_prepare_header() {
  $params = array();

  if (module_exists('mimemail')) {
//    $params['plain'] = !$email['html'];
//    $params['plaintext'] = $email['html'] ? NULL : $email['message'];
    // Mime Mail requires this header or it will filter all text.
    $params['headers']['Content-Type'] = 'text/html; charset=UTF-8';
  }
  elseif ($character_set = variable_get('mass_contact_character_set', '')) {
    $params['headers']['Content-Type'] = 'text/plain; charset=' . $character_set . '; format=flowed';
  }

  return $params;
} // End of _mass_contact_prepare_header().


/**
 * Prepares the message's body.
 *
 * @param form_values
 *   A keyed array containing the values returned with the form.
 *
 * @return
 *   A keyed array containing the prepared body as two strings. There are two
 *   possible values:
 *     $body['plain']: The plain text body part is here.
 *     $body['html']: The HTML body part is here.
 */
function _mass_contact_prepare_bodies($form_values) {
  $body = '';

  if (module_exists('mimemail')) {
    // Start with the message prefix.
    $body .= _mass_contact_prepare_message_addon('prefix', 'html');

    global $language;

    // Add in the actual message.
    if (is_array($form_values['message'])) {
      $body .= check_markup($form_values['message']['value'], $form_values['message']['format'], ($language) ? $language->language : LANGUAGE_NONE);
    }
    else {
      $body .= check_markup($form_values['message'], $form_values['body_filter_format']->format, ($language) ? $language->language : LANGUAGE_NONE);
    }

    // End with the message suffix.
    $body .= _mass_contact_prepare_message_addon('suffix', 'html');
  }
  // Mime Mail is not installed, so this message will only be processed as
  // plain text.
  else {
    // Start with the message prefix.
    $body .= _mass_contact_prepare_message_addon('prefix', 'converted');

    // Add in the actual message.
    if (is_array($form_values['message']['value'])) {
      $body .= wordwrap(check_plain($form_values['message']['value'])) . "\n";
    }
    else {
      $body .= wordwrap(check_plain($form_values['message'])) . "\n";
    }

    // End with the message suffix.
    $body .= _mass_contact_prepare_message_addon('suffix', 'converted');
  }

  return $body;
} // End of _mass_contact_prepare_bodies().


/**
 * Prepares a message addon (prefix or suffix).
 *
 * @param add_on
 *   A string specifing the addon part to process. There are two possible
 *   values:
 *     'prefix'
 *     'suffix'
 * @param style
 *   A string indicating how to process the addon. There are three possible
 *   values:
 *     'plain text': Indicates that the returned addon should be in plain
 *       text format.
 *     'html': Indicates that the returned addon should be in HTML format.
 *     'converted': Indicates that the returned addon is HTML but should be
 *       converted into plain text format.
 *
 * @return
 *   A string containing the prepared message addon.
 */
function _mass_contact_prepare_message_addon($addon, $style) {
  $token_module_exists = module_exists('token');
  $message = '';

  switch ($addon) {
    case 'prefix':
      if ($token_module_exists) {
        $message_addon = variable_get('mass_contact_message_prefix', t('[current-user:name] has sent you a group e-mail from [site:name].'));
      }
      else {
        $message_addon = variable_get('mass_contact_message_prefix', t('You were sent a group e-mail from @site.', array('@site' => url(NULL, array('absolute' => TRUE)))));
      }
      break;
    case 'suffix':
      if ($token_module_exists) {
        $message_addon = variable_get('mass_contact_message_suffix', '');
      }
      else {
        $message_addon = variable_get('mass_contact_message_suffix', '');
      }
      break;
  }

  if (is_array($message_addon)) {
    $message_addon_text = $message_addon['value'];
  }
  else {
    $message_addon_text = $message_addon;
  }

  if (!empty($message_addon_text)) {
    switch ($style) {
      case 'plain text':
        if ($token_module_exists) {
          $message .= wordwrap(check_plain(token_replace($message_addon_text)));
        }
        else {
          $message .= wordwrap(check_plain($message_addon_text));
        }
        break;

      case 'html':
        global $language;
        if (is_array($message_addon)) {
          if ($token_module_exists) {
            $message .= check_markup(token_replace($message_addon_text), $message_addon['format'], ($language) ? $language->language : LANGUAGE_NONE);
          }
          else {
            $message .= check_markup($message_addon_text, $message_addon['format'], ($language) ? $language->language : LANGUAGE_NONE);
          }
        }
        else {
          if ($token_module_exists) {
            $message .= check_markup(token_replace($message_addon_text), filter_fallback_format(), ($language) ? $language->language : LANGUAGE_NONE);
          }
          else {
            $message .= check_markup($message_addon_text, filter_fallback_format(), ($language) ? $language->language : LANGUAGE_NONE);
          }
        }
        break;

      case 'converted':
        if ($token_module_exists) {
          $message .= drupal_html_to_text(token_replace($message_addon_text));
        }
        else {
          $message .= drupal_html_to_text($message_addon_text);
        }
        break;
    }
  }

  return $message;
} // End of _mass_contact_prepare_message_addon().


/**
 * Process attachments for use with Mime Mail.
 *
 * @param nodecc
 *   An boolean identifying whether a copy of the message will be saved as a
 *   node.
 *
 * @return
 *   The attachment information, as Mime Mail expects it.
 */
function _mass_contact_process_mime_mail_attachments($nodecc = FALSE) {
  // Initialize the variable.
  $files = array();

  // Loop through each possible attachment.
  for ($i = 1; $i <= variable_get('mass_contact_number_of_attachments', '3'); $i++) {
    // Check to see if an attachment exists.
    if ($_FILES['files']['size']['attachment_' . $i] > 0) {
      // If the message is to be saved as a node, we need to save the
      // attachment now so that it is not lost later.
      if ($nodecc) {
         // The name in the global $_FILES array where the information about
         // the attachment is stored.
        $file_source = 'attachment_' . $i;
        $file_validators = array();
        // Get the directory where attachments are to be saved.
        $file_destination = 'public://' . variable_get('mass_contact_attachment_location');
        $file_replace = FILE_EXISTS_RENAME;

        // Check to see that the directory exists.
        if (!file_prepare_directory($file_destination)) {
          // It does not, so create the directoty.
          drupal_mkdir($file_destination);
        }

        // Save the file to the Mass Contact attachments directory.
        $file = file_save_upload($file_source, $file_validators, $file_destination, $file_replace);

        // Check to see if there was an error during the save.
        if ($file === FALSE) {
          // There was an error, so report it.
          $message = 'There was an error uploading the "!attachment" attachment.';
          $variables = array('!attachment' => $_FILES['files']['name']['attachment_' . $i]);

          drupal_set_message(t($message, $variables), 'error');
          watchdog('mass_contact', $message, $variables, WATCHDOG_ERROR);
        }
        elseif ($file === NULL) {
          // Nothing happened, so say so.
          $message = 'The "!attachment" attachment was not uploaded.';
          $variables = array('!attachment' => $_FILES['files']['name']['attachment_' . $i]);

          drupal_set_message(t($message, $variables), 'error');
          watchdog('mass_contact', $message, $variables, WATCHDOG_ERROR);
        }
        // The file was saved, so now the information needs to be added to the
        // database.
        else {
          //Set the status of the uploaded file so it stays around for a while.
          $file->status = FILE_STATUS_PERMANENT;

          // Save the file object to the database.
          $file = file_save($file);

          // Check the state to determine if there was an error or not.
          if (!$file) {
            // If there was an error, report it.
            $message = 'There was an error saving the file object for "!attachment" to the database.';
            $variables = array('!attachment' => $_FILES['files']['name']['attachment_' . $i]);

            drupal_set_message(t($message, $variables), 'error');
            watchdog('mass_contact', $message, $variables, WATCHDOG_ERROR);
          }
          else {
            // Add the file object to the files collection array.
            $files[] = $file;
          }
        }
      }
      // Save the files in a temporary location.
      else {
        $files[] = array(
          'filepath' => $_FILES['files']['tmp_name']['attachment_' . $i],
          'uri' => $_FILES['files']['tmp_name']['attachment_' . $i],
          'filename' => $_FILES['files']['name']['attachment_' . $i],
          'filemime' => $_FILES['files']['type']['attachment_' . $i],
        );
      }
    }
  }

  return $files;
} // End of _mass_contact_process_mime_mail_attachments().


/**
 * Creates a list of users who have opted out of receiving messages.
 *
 * @param categories
 *   An object containing the list of categories the message is to.
 *
 * @return
 *   An array with the keys being the user ids and the values being 1 to
 *   indicate the users' desire to opt out of the current mailing.
 */
function _mass_contact_create_opt_out_list($categories) {
  $opt_out_users = array();

  // Check for opt-out unless overridden by admin.
  $optout_setting = variable_get('mass_contact_optout_d', 0);

  // Allow users to opt-out of mass e-mails:
  // 0 == 'No', 1 == 'Yes', 2 == 'Selected categories'.
  if ($optout_setting == 1 || $optout_setting == 2) {
    // Select all active users to determine who has opted out.
    $active_users = db_select('users', 'u')
      ->fields('u', array('uid'))
      ->condition('status', 0, '<>')
      ->execute();
    foreach ($active_users as $user) {
      // Load the user so that we may access his/her data.
      $account = user_load($user->uid);
      if ($optout_setting == 1 && $account->mass_contact_optout == 1) {
        // If the user opted out of all messages, add their user id to the
        // list.
        $opt_out_users[$user->uid] = 1;
      }
      elseif ($optout_setting == 2) {
        // Cycle through all the categories selected for this message.
        foreach ($categories as $category) {
          $optout_cid = 'mass_contact_optout_' . $category->cid;
          if ($account->$optout_cid == 1) {
            // If the user opted out of this particular category, add their
            // user id to the list.
            $opt_out_users[$user->uid] = 1;
          }
        }
      }
    }
  }

  return $opt_out_users;
} // End of _mass_contact_create_opt_out_list().


/**
 * Creates a list of recipients.
 *
 * @param roles
 *   An array of roles to get users from.
 * @param opt_out_users
 *   An array of user ids for users who've indicated they want to be excluded
 *   from mass mailings. This list can be obtained by calling the
 *   _mass_contact_create_opt_out_list() function.
 *
 * @return
 *   An array of users' e-mail addresses, keyed by their user name.
 */
function _mass_contact_create_recipient_list($roles, $opt_out_users = array()) {
  global $user;

  $recipient_list = array();

  foreach ($roles as $role) {
    // Get the list of users in this role.
    if ($role == 2) {
      $query = "SELECT u.uid FROM {users} u WHERE u.status <> 0";
      $recipient_ids = db_query($query);
    }
    else {
      $query = "SELECT ur.uid FROM {users_roles} ur LEFT JOIN {users} u ON ur.uid = u.uid WHERE ur.rid = :ur_rid AND u.status <> 0";
      $recipient_ids = db_query($query, array(':ur_rid' => $role));
    }
//    $recipient_ids = db_query($query, array(':ur_rid' => $role))->fetchObject();
//    $recipient_ids = db_select('users_roles', 'ur')
//      ->fields('ur', array('uid'))
//      ->leftJoin('users', 'u', 'ur.uid = u.uid')
//      ->condition('rid', $role)
//      ->condition('status', 0, '<>')
//      ->execute();

    foreach ($recipient_ids as $recipient_id) {
      $recipient = db_select('users', 'u')
        ->fields('u', array('name', 'mail'))
        ->condition('uid', $recipient_id->uid)
        ->execute()
        ->fetchObject();

      if (isset($opt_out_users[$recipient_id->uid])) {
        if ($recipient->name != $user->name && !$opt_out_users[$recipient_id->uid]) {
          $recipient_list[$recipient->name] = $recipient->mail;
        }
      }
      else {
        if ($recipient->name != $user->name) {
          $recipient_list[$recipient->name] = $recipient->mail;
        }
      }
    }
  }

  return $recipient_list;
} // End of _mass_contact_create_recipient_list().


/**
 * Finalizes the headers, by setting the To, Subject, and possibly, Bcc fields.
 *
 * @param boolean $bcc
 *   Indicates whether the recipients are to be put into the To or Bcc field.
 *   TRUE indicates the Bcc field, and FALSE indicates the To field.
 * @param string $recipients
 *   The list of recipients, separated by a comma.
 * @param array &$params
 *   The message array.
 * @param string $from_email
 *   The message sender's e-mail address.
 * @param string $subject
 *   The subject of the message.
 *
 * @return string
 *   The contents of the To field. If $bcc is TRUE, this will be $from_email.
 *   If it is FALSE, it will be $recipients.
 *
 */
function _mass_contact_finalize_headers($bcc, $recipients, &$params, $from_email, $subject) {
  // If blind carbon copy (BCC) was selected, put the recipients in
  // the Bcc field.
  if ($bcc) {
    $params['headers']['Bcc'] = $recipients;
    $to = $from_email;
  }
  else {
    // Else, put the recipients in the To field.
    $to = $recipients;
  }

  $params['subject'] = $subject;

  return $to;
} // End of _mass_contact_finalize_headers().


/**
 * Sends the message.
 *
 * @param string $to
 *   To whom the message will be delivered.
 * @param array &$params
 *   The message array.
 * @param string $from_email
 *   The message sender's e-mail address.
 * @param string $success_message
 *   The message that will be displayed to the user, if sending is successful.
 * @param string $message_key
 *   A key to identify the e-mail sent.
 *
 * @return boolean
 *   The result of the call to drupal_mail():
 *     The $message array structure containing all details of the
 *     message. If already sent ($send = TRUE), then the 'result' element
 *     will contain the success indicator of the e-mail, failure being already
 *     written to the watchdog. (Success means nothing more than the message being
 *     accepted at php-level, which still doesn't guarantee it to be delivered.)
 */
function _mass_contact_send_message($message_key, $to, &$params, $from_email, $success_message) {
  $success = drupal_mail('mass_contact', $message_key, $to, language_default(), $params, $from_email);

  if ($success['result']) {
    drupal_set_message(t($success_message . ' (To: @e-mails.)', array('@e-mails' => (isset($params['headers']['Bcc'])) ? $params['headers']['Bcc'] : $to)));
  }

  return $success['result'];
} // End of _mass_contact_send_message().


/**
 * Archive a copy of the sent message as a node.
 *
 * @param array $params
 *   The message array.
 * @param string $body
 *   The body of the message and node.
 * @param array $fomat
 *   The text format the body is to use.
 * @param string $from
 *   The e-mail address representing who the message is from.
 * @param string $category
 *   The categories of users sent to.
 * @param string $recipients
 *   The users who received the message.
 * @param boolean $result
 *   The result of sending the message.
 */
function _mass_contact_save_node($params, $body, $format, $from, $category, $recipients, $result) {
  // Create a new node object.
  $node = new stdClass();

  // Set the node type.
  $node->type = 'mass_contact';

  // Prepare the node object for editing and fill in a few default values.
  node_object_prepare($node);

  // Set the language code if Locale module is enabled.
//  global $language;
//  $node->language = ($language) ? $language->language : LANGUAGE_NONE;
  // The node data is not completely saved if $node->language is not set to
  // LANGUAGE_NONE.
  // @TODO: Find out if this is a bug in Drupal or not.
  $node->language = LANGUAGE_NONE;

  // Set the title of the node to the subject of the message.
  $node->title = $params['subject'];

  // Set the body of the node to the body of the message.
  $node->body[$node->language][0]['value'] = $body;
  // Set the teaser or summary of the body.
  $node->body[$node->language][0]['summary'] = text_summary($body);
  // Set the text format of the message body.
  $node->body[$node->language][0]['format'] = $format;

  // Set the From field.
  $node->field_mass_contact_from[$node->language][0]['value'] = $from;

  // Set the Category field.
  $node->field_mass_contact_category[$node->language][0]['value'] = $category;

  // Set the To field.
  $to = explode(',', $recipients);
  foreach ($to as $recipient) {
    $node->field_mass_contact_to[$node->language][]['value'] = $recipient;
  }

  // Set the BCC field.
  if (!empty($params['headers']['Bcc'])) {
    $to = explode(',', $params['headers']['Bcc']);
    foreach ($to as $recipient) {
      $node->field_mass_contact_bcc[$node->language][]['value'] = $recipient;
    }
  }

  // Set the Headers field.
  if (!empty($params['headers'])) {
    // Format the output so that it is readable.
    $headers = '';
    foreach ($params['headers'] as $header => $value) {
      $headers .= $header . ": " . $value . "\n";
    }

    // Set the body of the Headers.
    $node->field_mass_contact_headers[$node->language][0]['value'] = $headers;
    // Set the teaser or summary of the Headers.
    $node->field_mass_contact_headers[$node->language][0]['summary'] = text_summary($headers);
    // Set the text format of the Headers.
    $node->field_mass_contact_headers[$node->language][0]['format'] = 'plain_text';
  }

  // Set the Result field.
  // Set the body of the Result.
  $node->field_mass_contact_result[$node->language][0]['value'] = $result;
  // Set the teaser or summary of the Result.
  $node->field_mass_contact_result[$node->language][0]['summary'] = text_summary($result);
  // Set the text format of the Result.
  $node->field_mass_contact_result[$node->language][0]['format'] = 'plain_text';

  // If any attachments exist, add them.
  if (!empty($params['attachments'])) {
    foreach ($params['attachments'] as $attachment) {
      // Attach the attachment to the node.
      $node->field_mass_contact_attachments[$node->language][] = array(
        'fid' => $attachment->fid,
        'display' => 1,
        'dscription' => '',
        'uid' => $attachment->uid,
        'filename' => $attachment->filename,
        'uri' => $attachment->uri,
        'filemime' => $attachment->filemime,
        'filesize' => $attachment->filesize,
        'status' => $attachment->status,
        'timestamp' => $attachment->timestamp,
        'rdf_mapping' => array(),
      );
    }
  }

  // Save the node.
  node_save($node);

  // Log the node creation.
  watchdog('content', 'Mass Contact content added "%title".', array('%title' => $params['subject']), WATCHDOG_NOTICE, l(t('View'), 'node/' . $node->nid));

  // Inform the user.
  drupal_set_message(t('A copy has been created as a node.'));
} // End of _mass_contact_save_node().
